@@ -30,7 +30,7 @@ from hopeit.server import api
 from hopeit.server.steps import find_datatype_handler
 from hopeit.toolkit import auth
 from hopeit.dataobjects.jsonify import Json
-from hopeit.app.context import EventContext, PostprocessHook
+from hopeit.app.context import EventContext, NoopMultiparReader, PostprocessHook, PreprocessHook
 from hopeit.dataobjects import DataObject, EventPayloadType
 from hopeit.app.errors import Unauthorized, BadRequest
 from hopeit.server.engine import Server, AppEngine
@@ -203,6 +203,12 @@ def _setup_app_event_routes(app_engine: AppEngine,
                     app_engine, plugin=plugin, event_name=event_name, override_route_name=event_info.route
                 )
             ])
+        elif event_info.type == EventType.MULTIPART:
+            web_server.add_routes([
+                _create_multipart_event_route(
+                    app_engine, plugin=plugin, event_name=event_name, override_route_name=event_info.route
+                )
+            ])
         elif event_info.type == EventType.STREAM and plugin is None:
             web_server.add_routes(
                 _create_event_management_routes(
@@ -269,6 +275,28 @@ def _create_get_event_route(
     return web.get(route, api_handler)
 
 
+def _create_multipart_event_route(
+        app_engine: AppEngine, *,
+        plugin: Optional[AppEngine] = None,
+        event_name: str,
+        override_route_name: Optional[str]) -> web.RouteDef:
+    """
+    Creates route for handling MULTIPART event
+    """
+    datatype = find_datatype_handler(app_config=app_engine.app_config, event_name=event_name)
+    route = app_route_name(app_engine.app_config.app, event_name=event_name,
+                           plugin=None if plugin is None else plugin.app_config.app,
+                           override_route_name=override_route_name)
+    logger.info(__name__, f"MULTIPART path={route} input={str(datatype)}")
+    impl = plugin if plugin else app_engine
+    handler = partial(_handle_multipart_invocation, app_engine, impl,
+                      event_name, datatype, _auth_types(impl, event_name))
+    setattr(handler, '__closure__', None)
+    setattr(handler, '__code__', _handle_multipart_invocation.__code__)
+    api_handler = api.add_route('post', route, handler)
+    return web.post(route, api_handler)
+
+
 def _create_event_management_routes(
         app_engine: AppEngine, *,
         event_name: str,
@@ -449,15 +477,20 @@ async def _request_execute(
         event_name: str,
         context: EventContext,
         query_args: Dict[str, Any],
-        payload: Optional[EventPayloadType] = None) -> ResponseType:
+        payload: Optional[EventPayloadType],
+        preprocess_hook: PreprocessHook) -> ResponseType:
     """
     Executes request using engine event handler
     """
-    result = await app_engine.execute(
-        context=context, query_args=query_args, payload=payload)
     response_hook = PostprocessHook()
-    result = await app_engine.postprocess(context=context, payload=result,
-                                          response=response_hook)
+    result = await app_engine.preprocess(
+        context=context, query_args=query_args, payload=payload, request=preprocess_hook)
+    if (preprocess_hook.status is None) or (preprocess_hook.status == 200):
+        result = await app_engine.execute(
+            context=context, query_args=query_args, payload=result)
+        result = await app_engine.postprocess(context=context, payload=result, response=response_hook)
+    else:
+        response_hook.set_status(preprocess_hook.status)
     body = Json.to_json(result, key=event_name)
     response = _response(track_ids=context.track_ids, body=body, hook=response_hook)
     logger.done(context, extra=combined(
@@ -475,10 +508,10 @@ async def _request_process_payload(
     Returns payload if parsing succeeded. Raises BadRequest if payload fails to parse
     """
     try:
-        payload_str = (await request.read()).decode()
-        if (payload_str is None) or (payload_str == ''):
+        payload_raw = await request.read()
+        if (payload_raw is None) or (payload_raw == b''):
             return None
-        payload = Json.from_json(payload_str, datatype) if datatype else payload_str
+        payload = Json.from_json(payload_raw, datatype) if datatype else payload_raw.decode()
         return payload  # type: ignore
     except ValueError as e:
         logger.error(context, e)
@@ -501,7 +534,8 @@ async def _handle_post_invocation(
         query_args = dict(request.query)
         _validate_authorization(app_engine.app_config, context, auth_types, request)
         payload = await _request_process_payload(context, datatype, request)
-        return await _request_execute(impl, event_name, context, query_args, payload)
+        hook: PreprocessHook[NoopMultiparReader] = PreprocessHook(headers=request.headers)
+        return await _request_execute(impl, event_name, context, query_args, payload, preprocess_hook=hook)
     except Unauthorized as e:
         return _ignored_response(context, 401, e)
     except BadRequest as e:
@@ -527,7 +561,9 @@ async def _handle_get_invocation(
         payload = query_args.get('payload')
         if payload is not None:
             del query_args['payload']
-        return await _request_execute(impl, event_name, context, query_args, payload=payload)
+        hook: PreprocessHook[NoopMultiparReader] = PreprocessHook(headers=request.headers)
+        return await _request_execute(impl, event_name, context, query_args, payload=payload,
+                                      preprocess_hook=hook)
     except Unauthorized as e:
         return _ignored_response(context, 401, e)
     except BadRequest as e:
@@ -536,6 +572,33 @@ async def _handle_get_invocation(
         return _failed_response(context, e)
 
 
+async def _handle_multipart_invocation(
+        app_engine: AppEngine,
+        impl: AppEngine,
+        event_name: str,
+        datatype: Optional[Type[DataObject]],
+        auth_types: List[AuthType],
+        request: web.Request) -> ResponseType:
+    """
+    Handler to execute POST calls
+    """
+    context = None
+    try:
+        context = _request_start(app_engine, impl, event_name, request)
+        query_args = dict(request.query)
+        _validate_authorization(app_engine.app_config, context, auth_types, request)
+        hook = PreprocessHook(                                                   # type: ignore
+            headers=request.headers, multipart_reader=await request.multipart()  # type: ignore
+        )
+        return await _request_execute(impl, event_name, context, query_args, payload=None,
+                                      preprocess_hook=hook)
+    except Unauthorized as e:
+        return _ignored_response(context, 401, e)
+    except BadRequest as e:
+        return _ignored_response(context, 400, e)
+    except Exception as e:  # pylint: disable=broad-except
+        return _failed_response(context, e)
+
 async def _start_streams(app_engine: AppEngine, scheduler: Scheduler):
     """
     Start all stream event types configured in app.
